/*
 * Copyright 2000 by Jean-Claude Christophe <jch@oleane.net>
 *                                          <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * main.c  Core program.
 */

#include <sys/types.h>
#include <sys/socket.h> /* shutdown(2) */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "draft2.h"
#include "tac_bsd.h"
#include "conf.h"

/* in connect.c */
extern int connect_to_srv(char *);

/* in packet.c */
extern int send_authen(int, char*, char *, char *);
extern int send_auth_cont(int, char *, int);
extern int read_authen(int);
extern int read_result(int);

/* in login.c */
extern void login();
extern char *stypeof(char *);

/* in misc.c */
extern void authenticate(void);

char *username; /* username we will use for authentication */
char *password; /* idem for password */
char *port;     /* current tty */
char *term;     /* our teminal info */

int try = 0;

void
display_file(file)
    char *file;
{
    FILE *fp;
    char c[256];
    
    fp = fopen(file, "r"); /* we don't check, read_conf() already did it. */
    while (fgets(c, 256, fp))
	printf("%s", c);
}

void
close_quit()
{
    int desc;

    for (desc = getdtablesize(); desc > 2; desc--) {
	shutdown(desc, SHUT_RDWR);
	close(desc);
    }
    exit(-1);
}

int
main(argc, argv)
    int argc;
    char *argv[];
{
    int result, done = 0;
    int srv_sd;   /* socket used by the server */
    

    if (!read_conf(CONF_FILE)) {
	perror(CONF_FILE);
	exit(-1);
    }
    port = ttyname(STDIN_FILENO); /* What is the current tty ? it will be our port.
				   * STDIN_FILENO = 0
				   */
    term = stypeof(port + 5);     /* +5 for remove the "/dev" */
    
    username = (char *)malloc(256);
    if (!username) { /* not enough memory */
	perror("username");
	exit(-1);
    }
    
    /* display motd if set */
    if (premotd) display_file(premotd);
    
    printf("User Access Verification\n\n"); /* cisco's style banner */

    while ((try != maxtry) && !done) {
	/* now, it's time to be serious.
	 * Our authentication can be resumed into 4 steps (we use TYPE_ASCII):
	 * 1) after established the connection, we send the START request
	 * 2) we are waiting for the REPLY
	 * 3) If the REPLY seems to be correct, we send the CONTINUE packet
	 * 4) The daemon reply by PASS or FAIL: the result we want.
	 */

	/* ask for username and password */

	authenticate();
	
	/* First step: connection and START request */
	srv_sd = connect_to_srv(server);
	
	if (srv_sd < 0) 
	    exit(srv_sd);
	
	if (send_authen(srv_sd, username, password, port) < 0) {
	    if (slog) 
		fprintf(stderr, "Error in first step: check into syslog for details\n");
	    close_quit();
	}
	
	/* Second step: wait the response, we're expecting a GETPASS */
	if (read_authen(srv_sd) < 0) {
	    if (slog) 
		fprintf(stderr, "Error in second step: check into syslog for details\n");
	    close_quit();
	}
	
	/* Third step: send the CONTINUE packet */
	if (send_auth_cont(srv_sd, password, strlen(password)) < 0) {
	    if (slog) 
		fprintf(stderr, "Error in third step: check into syslog for details\n");
	    close_quit();
	}
	
	/* Fourth step: get the fatal result: PASS or FAIL */
	result = read_result(srv_sd);
	if (result < 0) {
	    if (slog) 
		fprintf(stderr, "Error in fourth step: check into syslog for details\n");
	    close_quit();
	} else if (result) {
	    try++;
	    printf("\nAuthentication failed.\n\n");
	} else 
	    done = 1; /* done */
	shutdown(srv_sd, SHUT_RDWR); close(srv_sd);
    }
    
    if (try == maxtry) {
	if (motdfail) {
	    display_file(motdfail);
	    close_quit();
	}
    } else if (motdpass) { /* all is good, we accept the user to log into this machine */
	display_file(motdpass);
	login();
    }
    
    return(0);
}
